package coolCalendar;

import javax.swing.event.EventListenerList;

/**
 * This provides an abstract base class for all CalendarModels to
 * implement so they don't have to code their own listener handlers;
 * this class does not have to be used, it is provided for convenience.
 */
public abstract class AbstractCalendarModel implements CalendarModel
{
    private EventListenerList listenerList = new EventListenerList();

    public void addCalendarModelListener(CalendarModelListener l)
    {
        listenerList.add(CalendarModelListener.class, l);
    }

    public void removeCalendarModelListener(CalendarModelListener l)
    {
        listenerList.remove(CalendarModelListener.class, l);
    }

    protected void fireCalendarModelAdded(Appointable appointable)
    {
        CalendarModelEvent calendarModelEvent = null;

        // Guaranteed to return a non-null array

        Object[] listeners = listenerList.getListenerList();

        // Process the listeners last to first, notifying
        // those that are interested in this event

        for (int i = listeners.length - 2; i >= 0; i -= 2)
        {
            if (listeners[i] == CalendarModelListener.class)
            {
                // Lazily create the event:

                if (calendarModelEvent == null)
                    calendarModelEvent = new CalendarModelEvent(this, appointable);

                ((CalendarModelListener) listeners[i+1]).appointmentAdded(calendarModelEvent);
            }
        }
    }

    protected void fireCalendarModelRemoved(Appointable appointable)
    {
        CalendarModelEvent calendarModelEvent = null;

        // Guaranteed to return a non-null array

        Object[] listeners = listenerList.getListenerList();

        // Process the listeners last to first, notifying
        // those that are interested in this event

        for (int i = listeners.length - 2; i >= 0; i -= 2)
        {
            if (listeners[i] == CalendarModelListener.class)
            {
                // Lazily create the event:

                if (calendarModelEvent == null)
                    calendarModelEvent = new CalendarModelEvent(this, appointable);

                ((CalendarModelListener) listeners[i+1]).appointmentRemoved(calendarModelEvent);
            }
        }
    }

    protected void fireCalendarModelUpdated(Appointable appointable)
    {
        CalendarModelEvent calendarModelEvent = null;

        // Guaranteed to return a non-null array

        Object[] listeners = listenerList.getListenerList();

        // Process the listeners last to first, notifying
        // those that are interested in this event

        for (int i = listeners.length - 2; i >= 0; i -= 2)
        {
            if (listeners[i] == CalendarModelListener.class)
            {
                // Lazily create the event:

                if (calendarModelEvent == null)
                    calendarModelEvent = new CalendarModelEvent(this, appointable);

                ((CalendarModelListener) listeners[i+1]).appointmentUpdated(calendarModelEvent);
            }
        }
    }
}
