package coolCalendar;

import java.util.*;
import java.io.*;

abstract class Appointment
    implements Comparable, Appointable, Serializable
{
    // a unique key
    private Integer key=null;

    public void setKey(Integer nwk)
    {
	key = nwk;
    }

    public Integer getKey()
    {
	return key;
    }

    // a key counter
    static private int key_ctr = 0;

    // the description of the event
    private String description=null;

    public void setDescription(String descr)
    {
	description = descr;
    }

    public String getDescription()
    {
	return description;
    }

    // the start date of the event
    private Date startDate=null;

    /** set the start date (and time) to a copy of the given
	argument.
     */
    public void setStartDate(Date startD)
    {
	startDate=new Date(startD.getTime());
    }

    /** returns the internal start date (and time).
     */
    public Date getStartDate()
    {
	return startDate;
    }

    // the duration of one event
    private int duration;
    
    public void setDuration(int dur)
    {
	duration = dur;
    }
    
    public int getDuration()
    {
	return duration;
    }

    // the advance reminder's time
    private int reminderTime;
    
    public void setReminderTime(int dur)
    {
	reminderTime = dur;
    }

    public int getReminderTime()
    {
	return reminderTime;
    }

    // the unit in which the reminder time is specified
    // 
    private int reminderTimeU;

    /** returns the unit of the reminder as MINUTE,DAY_OF_YEAR,
	of the class java.util.Calendar.

	@see java.util.Calendar
     */
    int getReminderTimeUnit()
    {
	return reminderTimeU;
    }

    /** sets the unit of the reminder as MINUTE,DAY_OF_YEAR,
	of the class java.util.Calendar.

	@see java.util.Calendar
     */
    int setReminderTimeUnit()
    {
	return reminderTimeU;
    }

    // the period
    private int periodTime;
    
    public void setPeriodTime(int dur)
    {
	periodTime = dur;
    }

    public int getPeriodTime()
    {
	return periodTime;
    }

    // the unit in which the period time is specified
    // 
    private int periodTimeU;

    /** returns the unit of the period as MINUTE,DAY_OF_YEAR,
	of the class java.util.Calendar.

	@see java.util.Calendar
     */
    int getPeriodTimeUnit()
    {
	return periodTimeU;
    }

    /** sets the unit of the period time MINUTE,DAY_OF_YEAR,
	of the class java.util.Calendar.

	@see java.util.Calendar
     */
    int setPeriodTimeUnit()
    {
	return periodTimeU;
    }

    // the end date of the last event in the period
    private Date periodEndTime=null;

    /** set the end date (and time) to a copy of the given
	argument.
     */
    public void setPeriodEndTime(Date endD)
    {
	periodEndTime=new Date(endD.getTime());
    }

    /** returns the internal end date (and time).
     */
    public Date getPeriodEndTime()
    {
	return periodEndTime;
    }



    private HashSet participantList=null;

    public HashSet getParticipantList()
    {
	return participantList; 
    }

    public void setParticipantList(HashSet pl)
    {
	participantList = pl; 
    }

	public boolean isShared(){
	return (participantList.size()>1);
	}

	public boolean hasUser(User user){
	return participantList.contains(user.getEmail());
	}


    /** two calendar events are compared according to their start date and their 
        end date. If start and end date are the same, the unique key is used for 
        comparison to ensure a total order.
     */
    public int compareTo(Object o)
        throws ClassCastException 
    {
        Appointment other = (Appointment) o;

        // compare the start date
        int start = startDate.compareTo(other.startDate );
        
        if (start == 0)
            { // compare the end date
                int end = getEndDate().compareTo(other.getEndDate());
                if (end == 0)
                    { // now use the unique keys
                        return key.compareTo(other.getKey());
                    }
                else
                    { // use the end date
                        return end;
                    }
            }
        else
            { // use the start date
                return start;
            }
    }

    /** stores the parameters and create a unique key.
     */
    protected Appointment(Date startD, String descr, int remind, int remindU
    	, int period, int periodU, Date periodEnd, HashSet plist)
    {
	key = new Integer(key_ctr++);
	startDate = startD;
	description = descr;
	reminderTime = remind;
	reminderTimeU = remindU;

	periodEndTime = periodEnd;
	periodTime = period;
	periodTimeU = periodU;
	participantList = plist;
    }

    /** computes the end date of this event.
	@return null if no duration is set
     */
    public Date getEndDate()
    {
	GregorianCalendar cal = new GregorianCalendar();
	cal.setTime(startDate);
	cal.add(getDurationUnit(), duration);
// 	System.out.println("from SD "+startDate+" +duration "+duration+" unit="+getDurationUnit()+
// 			   " (HOUR="+Calendar.HOUR_OF_DAY+") "+
// 			   " comp. "+cal.getTime());
	
	return cal.getTime();
    }

    /** returns the unit of the duration. Use HOUR_OF_DAY,MINUTE,DAY_OF_YEAR,
	WEEK_OF_YEAR of the class java.util.Calendar.

	@see java.util.Calendar
     */
    abstract int getDurationUnit();
}
