package coolCalendar;

import java.util.*;

public class MutableCalendarModel 
    extends AbstractCalendarModel
    implements CalendarModel
{

    protected CalendarDB myDB= null;

    /** creates a mutable calendar model for a given calendar data base.
     */
    public MutableCalendarModel(CalendarDB cdb)
    {
	myDB = cdb;
    }

    public boolean hasAppointments(Date from, Date to)
    {
        return myDB.hasAppointments(from,to);
    }

    public Appointable[] getAppointments(Date from, Date to)
    {
        return myDB.getAppointments(from,to);
    }

    public Appointable[] getReminders(Date date)
    {
        return myDB.getReminders(date);
    }

    /**
       creates a new open appointment and adds it to the calendar.
       @return the identifier of the object (a positive integer)
     */
    Integer addOpenAppointment(OpenAppointment oapp)
    {
	Integer key = myDB.addOpenAppointment(oapp);
	if (key.intValue() != -1)
	    { // the appointment is actually added
		fireCalendarModelAdded(getAppointment(key));
	    }
	return key;
    }

    /**
       creates a new closed appointment and adds it to the calendar.

       @return the positive identifier of the object or -1 (wrapped as Integer) 
               if this event would overlap with another closed event.
     */
    Integer addClosedAppointment(ClosedAppointment cevt)
    {
	Integer key = myDB.addClosedAppointment(cevt);
	if (key.intValue() != -1)
	    { // the appointment is actually added
		fireCalendarModelAdded(getAppointment(key));
	    }
	return key;
    }

    /** returns an open or closed appointment for a given key. 
	@return a copy or null if there is no appointment for the key.
     */
    Appointable getAppointment(Integer key)
    {
	return myDB.getAppointment(key);
    }

    /** deletes an appointment with a given key. 

	@return true if the key exists in the data base otherwise false
     */
    public boolean deleteAppointment(Integer key)
    {
	Appointable app = getAppointment(key);
	boolean res = myDB.deleteAppointment(key);
	if (res)
	    {
		fireCalendarModelRemoved(app);
	    }
	return res;
    }

    /** updates an appointment from another appointment with the same
	key. This can convert an open appointment to a closed and vice
	versa.
	
	@return true if there is an appointment with this key, otherwise false.
    */
    boolean updateAppointment(Appointment app)
    {
	boolean res = myDB.updateAppointment(app);
	if (res)
	    {
		fireCalendarModelUpdated(getAppointment(app.getKey()));
	    }
	return res;
    }

}







