package coolCalendar;

import java.util.*;
import java.text.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.border.*;

/**
 * This class provides the main GUI functionality for the calendar
 * program.
 */
public class MutableCalendarPanel extends JPanel
     implements ChangeListener, ItemListener, CalendarModelListener
{
    // Calendar related stuff
    private MultiCalendarModel model = null;

    // GUI stuff
    private CalendarPanel calendarPanel = null;
    private JTextField reminderDateField = null;
    private JButton reminderButton = null;
    private JButton todayButton = null;
    private Timer reminderTimer = null;

    private JComboBox yearCombo = null;
    private JComboBox monthCombo = null;

    private JButton addButton = null;
    private JButton editButton = null;
    private JButton removeButton = null;

    private JLabel selectedLabel = null;
    private JList list = null;

    private Calendar currentCalendar = null;
    private DateFormat dateFormat = null;

    private UserDB userDB;


    /**
     * Constructor for the frame, creates all components
     * and the database.
     *
     * @param calmod the calendar model
     */
    public MutableCalendarPanel(MultiCalendarModel model, UserDB udb)
    {
        // Save model.
        this.model = model;
        model.addCalendarModelListener(this);
	userDB = udb;

        // Get instance of date formatter.
        dateFormat = DateFormat.getDateInstance(DateFormat.SHORT);
        reminderTimer = new Timer( 10000, new ActionListener() {
            public void actionPerformed(ActionEvent event)
            {
	            Calendar calendar = calendarPanel.getCalendar();
    	        calendar.setTime(getReminderDate());
            	calendarPanel.setCalendar(calendar);
            } });
        reminderTimer.setRepeats(false);

        // Set layout manager.
        setLayout(new BorderLayout());

        // Add the year and month chooser.
        JPanel northPanel = new JPanel();
        add(northPanel, BorderLayout.NORTH);

		northPanel.add(reminderDateField = new JTextField(11), BorderLayout.WEST);
		reminderDateField.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent event)
            {
	            Calendar calendar = calendarPanel.getCalendar();
    	        calendar.setTime(getReminderDate());
            	calendarPanel.setCalendar(calendar);
            } });

		northPanel.add(reminderButton = new JButton("Reminders"), BorderLayout.WEST);
		reminderButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent event)
            {
				getReminders();
            } });

		northPanel.add(todayButton = new JButton("Today"), BorderLayout.WEST);
		todayButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent event)
            {
	            Calendar calendar = calendarPanel.getCalendar();
    	        calendar.setTime(new Date());
            	calendarPanel.setCalendar(calendar);
            } });

        // Add the year chooser.
        northPanel.add(yearCombo = new JComboBox(), BorderLayout.WEST);
        int currentYear = Calendar.getInstance().get(Calendar.YEAR);
		for(int i=-1;i<101;i++)
	        yearCombo.addItem(new Integer(currentYear+i).toString());
        yearCombo.addItemListener(this);

        // Add the month chooser.
        northPanel.add(monthCombo = new JComboBox(), BorderLayout.WEST);
        monthCombo.addItem("January");
        monthCombo.addItem("February");
        monthCombo.addItem("March");
        monthCombo.addItem("April");
        monthCombo.addItem("May");
        monthCombo.addItem("June");
        monthCombo.addItem("July");
        monthCombo.addItem("August");
        monthCombo.addItem("September");
        monthCombo.addItem("October");
        monthCombo.addItem("November");
        monthCombo.addItem("December");
        monthCombo.addItemListener(this);

        // Add the calendar panel.
        Calendar today = Calendar.getInstance();
        calendarPanel = new CalendarPanel(today, model);
        calendarPanel.addChangeListener(this);
        add(calendarPanel, BorderLayout.CENTER);

        // Add the edit panel.
        add(createEditPanel(), BorderLayout.EAST);

	

        // Call stateChanged() to make sure
        // the GUI is properly initialized.
        stateChanged(new ChangeEvent(calendarPanel));
        getReminders();
    }

    /**
     * Return the date of the reminder.
     */
    public Date getReminderDate()
    {
        Date date = null;
        DateFormat dateFormat = DateFormat.getInstance();
        // Parse the date.
        try {
            date = dateFormat.parse(reminderDateField.getText());
        } catch (ParseException ex) {
            date = new Date();
        }

        return date;
    }

    /**
     * Set the date of the reminder.
     */
    public void setReminderDate(Date date)
    {
        DateFormat dateFormat = DateFormat.getInstance();
        reminderDateField.setText(dateFormat.format(date));
    }

	protected void getReminders() {

		Appointable[] appointables = null;

		if (model != null)
			appointables = model.getReminders(getReminderDate());

		if (appointables == null)
			appointables = new Appointable[0];

        DateFormat dateFormat = DateFormat.getInstance();
        selectedLabel.setText("Reminders at "+
        	dateFormat.format(getReminderDate()));

		list.setListData(appointables);
		reminderTimer.start();
	}

    /**
     * Creates GUI components for the edit panel.
     */
    protected JPanel createEditPanel()
    {
        JPanel buttonPanel = new JPanel(new GridLayout(1, 3));
        buttonPanel.add(addButton = new JButton("Add"));
        buttonPanel.add(editButton = new JButton("Edit"));
        buttonPanel.add(removeButton = new JButton("Remove"));

        JPanel buttonAndLabelPanel = new JPanel(new GridLayout(2, 1));
        buttonAndLabelPanel.add(buttonPanel);
        buttonAndLabelPanel.add(selectedLabel = new JLabel("", JLabel.CENTER));


        JPanel panel = new JPanel(new BorderLayout());
        panel.add(buttonAndLabelPanel, BorderLayout.NORTH);
		list = new JList(){ public void paintComponent(Graphics g) {
			if (g != null) {
				String dayText = new String();

				switch (calendarPanel.getCalendar().get(Calendar.DAY_OF_WEEK)) {

				case Calendar.MONDAY:
					dayText = new String("Monday");
					break;
				case Calendar.TUESDAY:
					dayText = new String("Tuesday");
					break;
				case Calendar.WEDNESDAY:
					dayText = new String("Wednesday");
					break;
				case Calendar.THURSDAY:
					dayText = new String("Thursday");
					break;
				case Calendar.FRIDAY:
					dayText = new String("Friday");
					break;
				case Calendar.SATURDAY:
					dayText = new String("Saturday");
					break;
				case Calendar.SUNDAY:
					dayText = new String("Sunday");
					break;
				}

				String dayImage = new String();
				dayImage = Definition.ImagesDir + "/" + dayText.toLowerCase() + ".jpg";
				ImageIcon tmpImage = new ImageIcon(dayImage);
				g.drawImage(tmpImage.getImage(), 0, 0, list.getWidth(), list.getHeight(), this);
				g.setColor(Color.red);
				g.drawString(dayText, 10, list.getHeight()-20);
				super.paintComponent(g);
			}}};
		list.setOpaque(false);
	    list.setCellRenderer(new MyListCellRenderer());

		MouseListener mouseListener = new MouseAdapter() {
		public void mouseClicked(MouseEvent e) {
			if (e.getClickCount() == 2) {
				int index = list.locationToIndex(e.getPoint());
				editAppointment();
				}
			}
		};
		list.addMouseListener(mouseListener);

        panel.add(new JScrollPane(list), BorderLayout.CENTER);

	



        // Add action listeners.
        addButton.addActionListener(new ActionListener() {
            public void checkOverlap(Integer key)
            {
                    // Check for overlap.
                    if (key.intValue() == -1)
                    {
                        JOptionPane.showMessageDialog(MutableCalendarPanel.this,
                            "This appointment does overlap with some other appointment!"+
                            "\nThe record has NOT been created!",
                            "Invalid Input Data", JOptionPane.ERROR_MESSAGE);
                    }
			}

            public void actionPerformed(ActionEvent event)
            {
		//stop the Timer used for deleting the Reminders JList
		reminderTimer.stop();

                // Show appointment dialog.
                AppointmentDialog dlg = new AppointmentDialog(
		       JOptionPane.getFrameForComponent(MutableCalendarPanel.this),
                       userDB, model.getUser());

		HashSet pl=new HashSet();
		pl.add(model.getUser().getEmail());
                centerWindow(dlg);

		Calendar calendar = calendarPanel.getCalendar();

                dlg.setDate(calendar.getTime());
                dlg.setPeriodUnit(Calendar.YEAR);
                calendar.add(Calendar.MINUTE, 1);
                dlg.setPeriodEnd(calendar.getTime());
		dlg.setParticipantList(pl);
                dlg.show();

		calendar = calendarPanel.getCalendar();
		if (!(dlg.getPeriodicAppointment()))
		    dlg.setPeriod(0);

                // If the user clicked okay, then try to add
                // the appointment to the model.
                if (dlg.isOkay())
                {

                    Integer key = null;
                    Appointment app = null;
                    if (dlg.getType() == dlg.OPEN_TYPE)
			{
			    calendar.setTime(dlg.getDate());
			    do {
	                        key = model.addOpenAppointment(
					   new OpenAppointment(calendar.getTime(),
							       dlg.getDescription(),
							       dlg.getDuration(),
							     dlg.getReminderTime(), 
							  dlg.getReminderTimeUnit(),
							       dlg.getPeriod(),
							       dlg.getPeriodUnit(), 
							       dlg.getPeriodEnd(),
							 dlg.getParticipantList()));
				checkOverlap(key);
				calendar.add(dlg.getPeriodUnit(), dlg.getPeriod());
			    } while (dlg.getPeriodicAppointment() &&
				    calendar.getTime().before(dlg.getPeriodEnd()));
			}
		    else
			{
			    calendar.setTime(dlg.getDate());
			    do {
	                        key = model.addClosedAppointment(
					   new ClosedAppointment(calendar.getTime(),
							       dlg.getDescription(),
							       dlg.getDuration(),
							       dlg.getDurationUnit(),
							      dlg.getReminderTime(),
							  dlg.getReminderTimeUnit(),
							      dlg.getPeriod(), 
								dlg.getPeriodUnit(), 
								 dlg.getPeriodEnd(),
							 dlg.getParticipantList()));
				checkOverlap(key);
//				System.out.println("Key: "+key);

				calendar.add(dlg.getPeriodUnit(), dlg.getPeriod());
			    } while (dlg.getPeriodicAppointment() &&
				     calendar.getTime().before(dlg.getPeriodEnd()));
			}
                }
            }
        });

        editButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent event)
            {
            	editAppointment();
            }
        });

        removeButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent event)
            {
		//stop the Timer used for deleting the Reminders JList
		reminderTimer.stop();

                // Get selected appointment and remove it from model.
                Appointment app = (Appointment) list.getSelectedValue();
                if (app != null)
                {
                    model.deleteAppointment(app);
                }
            }
        });

        return panel;
    }

    public void editAppointment() {
        //stop the Timer used for deleting the Reminders JList
	reminderTimer.stop();

	// Get selected appointment.
	Appointment app = (Appointment) list.getSelectedValue();
	if (app != null)
	    {
		// Show appointment dialog.
		AppointmentDialog dlg = new AppointmentDialog(
		      JOptionPane.getFrameForComponent(MutableCalendarPanel.this),
		      userDB, model.getUser());
		centerWindow(dlg);
		dlg.setType((app instanceof OpenAppointment) ? 
			    dlg.OPEN_TYPE : dlg.CLOSED_TYPE);

		dlg.setDate(app.getStartDate());
		dlg.setDuration(app.getDuration());
		dlg.setDurationUnit(app.getDurationUnit());
		dlg.setDescription(app.getDescription());
		dlg.setReminderTime(app.getReminderTime());
		dlg.setReminderTimeUnit(app.getReminderTimeUnit());
		if (app.getPeriodTime() > 0) {
		    dlg.setPeriodicAppointment(true);
		    dlg.setPeriod(app.getPeriodTime());
		    dlg.setPeriodUnit(app.getPeriodTimeUnit());
		    dlg.setPeriodEnd(app.getPeriodEndTime());
		} else
		    dlg.setPeriodicAppointment(false);
		dlg.setParticipantList(app.getParticipantList());
		dlg.show();

		// If the user clicked okay, then try to update
		// the appointment in the model.
		if (dlg.isOkay())
                    {
                        Appointment newApp = null;
                        if (dlg.getType() == dlg.OPEN_TYPE)
			    {
				newApp = new OpenAppointment(dlg.getDate(),
                            	dlg.getDescription(), dlg.getDuration(),
                            	dlg.getReminderTime(), dlg.getReminderTimeUnit(),
                            	dlg.getPeriod(), dlg.getPeriodUnit(), 
				dlg.getPeriodEnd(),dlg.getParticipantList());
			    }
                        else
			    {
                            newApp = new ClosedAppointment(dlg.getDate(),
                                dlg.getDescription(), dlg.getDuration(),
                                dlg.getDurationUnit(),
                            	dlg.getReminderTime(), dlg.getReminderTimeUnit(),
                            	dlg.getPeriod(), dlg.getPeriodUnit(), 
				dlg.getPeriodEnd(),
				dlg.getParticipantList());
                        }

                        newApp.setKey(app.getKey());
                        if (!model.updateAppointment(newApp))
                        {
                            JOptionPane.showMessageDialog(MutableCalendarPanel.this,
                                "The edited appointment is not valid!",
                                "Invalid Input Data", JOptionPane.ERROR_MESSAGE);
                        }
                    }
                }
	}

    /**
     * ChangeListener interface; this listens for changes
     * to the CalendarPanel so it can update the year and month combo box
     * and the list of appointments.
     */
    public void stateChanged(ChangeEvent event)
    {
        CalendarPanel panel = (CalendarPanel) event.getSource();
        Calendar calendar = panel.getCalendar();

        if (panel.getSelectedDay() >= 0)
        {
            calendar.set(Calendar.DAY_OF_MONTH, panel.getSelectedDay());

            // Set the selected label.
            selectedLabel.setText("Appointments on "+dateFormat.format(calendar.getTime()));

            // Query for appointments.
            calendar.set(Calendar.HOUR_OF_DAY, 0);
            calendar.set(Calendar.MINUTE, 0);
            calendar.set(Calendar.SECOND, 0);
            Date from = calendar.getTime();
            calendar.set(Calendar.HOUR_OF_DAY, 23);
            calendar.set(Calendar.MINUTE, 59);
            calendar.set(Calendar.SECOND, 59);
            Date to = calendar.getTime();

            Appointable[] appointables = panel.getModel().getAppointments(from, to);

            if (appointables == null)
                appointables = new Appointable[0];
	    
            list.setListData(appointables);
        }

		// A check variable to update the MONTH & YEAR in the same time
        boolean updateCalendar = false;

        // Update the month combo box if necessary.
        calendar = panel.getCalendar();

        if ((currentCalendar == null) ||
            (calendar.get(Calendar.MONTH) != currentCalendar.get(Calendar.MONTH)))
        {
            int month = calendar.get(Calendar.MONTH);

            if (month == Calendar.JANUARY)
                monthCombo.setSelectedItem("January");
            else if (month == Calendar.FEBRUARY)
                monthCombo.setSelectedItem("February");
            else if (month == Calendar.MARCH)
                monthCombo.setSelectedItem("March");
            else if (month == Calendar.APRIL)
                monthCombo.setSelectedItem("April");
            else if (month == Calendar.MAY)
                monthCombo.setSelectedItem("May");
            else if (month == Calendar.JUNE)
                monthCombo.setSelectedItem("June");
            else if (month == Calendar.JULY)
                monthCombo.setSelectedItem("July");
            else if (month == Calendar.AUGUST)
                monthCombo.setSelectedItem("August");
            else if (month == Calendar.SEPTEMBER)
                monthCombo.setSelectedItem("September");
            else if (month == Calendar.OCTOBER)
                monthCombo.setSelectedItem("October");
            else if (month == Calendar.NOVEMBER)
                monthCombo.setSelectedItem("November");
            else if (month == Calendar.DECEMBER)
                monthCombo.setSelectedItem("December");

            updateCalendar = true;
        }

        if ((currentCalendar == null) || 
            (calendar.get(Calendar.YEAR) != currentCalendar.get(Calendar.YEAR)))
        {
            yearCombo.setSelectedItem(new Integer(calendar.get(Calendar.YEAR)).toString());

            updateCalendar = true;
        }

		if (updateCalendar)
            currentCalendar = calendar;

		setReminderDate(calendar.getTime());

    }

    /**
     * ItemListener interface; this listens for changes
     * in the month combo box and changes the date of the
     * CalendarPanel when a change occurs.
     */
    public void itemStateChanged(ItemEvent event)
    {
        if (event.getStateChange() == ItemEvent.SELECTED)
        {
            String monthString = (String) monthCombo.getSelectedItem();

            int month = -1;

            if (monthString.equals("January"))
                month = Calendar.JANUARY;
            else if (monthString.equals("February"))
                month = Calendar.FEBRUARY;
            else if (monthString.equals("March"))
                month = Calendar.MARCH;
            else if (monthString.equals("April"))
                month = Calendar.APRIL;
            else if (monthString.equals("May"))
                month = Calendar.MAY;
            else if (monthString.equals("June"))
                month = Calendar.JUNE;
            else if (monthString.equals("July"))
                month = Calendar.JULY;
            else if (monthString.equals("August"))
                month = Calendar.AUGUST;
            else if (monthString.equals("September"))
                month = Calendar.SEPTEMBER;
            else if (monthString.equals("October"))
                month = Calendar.OCTOBER;
            else if (monthString.equals("November"))
                month = Calendar.NOVEMBER;
            else if (monthString.equals("December"))
                month = Calendar.DECEMBER;
    
			Integer yearInteger = new Integer((String)yearCombo.getSelectedItem());

            Calendar calendar = calendarPanel.getCalendar();
            calendar.set(Calendar.MONTH, month);
            calendar.set(Calendar.YEAR, yearInteger.intValue());
            calendarPanel.setCalendar(calendar);
        }
    }

    /**
     * CalendarModelListener interfaces; the following interfaces
     * update the appointment list component when the data model
     * changes.
     */
    public void appointmentAdded(CalendarModelEvent event)
    {
        Calendar calendar = calendarPanel.getCalendar();

        Calendar c = Calendar.getInstance();
        c.setTime(event.getAppointment().getStartDate());

        if ((calendar.get(Calendar.MONTH) == c.get(Calendar.MONTH)) &&
            (calendarPanel.getSelectedDay() == c.get(Calendar.DAY_OF_MONTH)))
        {
            calendar.set(Calendar.DAY_OF_MONTH, c.get(Calendar.DAY_OF_MONTH));
    
            // Query for appointments.
            calendar.set(Calendar.HOUR_OF_DAY, 0);
            calendar.set(Calendar.MINUTE, 0);
            calendar.set(Calendar.SECOND, 0);
            Date from = calendar.getTime();
            calendar.set(Calendar.HOUR_OF_DAY, 23);
            calendar.set(Calendar.MINUTE, 59);
            calendar.set(Calendar.SECOND, 59);
            Date to = calendar.getTime();

            Appointable[] appointables = calendarPanel.getModel().getAppointments(from, to);

            if (appointables == null)
                appointables = new Appointable[0];
    
            list.setListData(appointables);
        }
    }

    public void appointmentRemoved(CalendarModelEvent event)
    {
        // This can be handled the same as added.
        appointmentAdded(event);
    }

    public void appointmentUpdated(CalendarModelEvent event)
    {
        // Since we really don't know the affect of update,
        // because it could be changing the date or whatever,
        // we will just simply reset the calendar panel.
        calendarPanel.setCalendar(calendarPanel.getCalendar());
    }

    /**
     * Utility function to center window.
     */
    public void centerWindow(Component window)
    {
        Toolkit toolkit = Toolkit.getDefaultToolkit();
        Dimension dim = toolkit.getScreenSize();
        int x = (dim.width - window.getSize().width) / 2;
        int y = (dim.height - window.getSize().height) / 2;
        window.setLocation(x, y);
    }
}
