package coolCalendar;

import java.util.*;
import java.io.*;
import javax.swing.filechooser.*;

/**
 * The database of appointments.
 */

class UserDB
{
    // some hashmaps to the same events
    private HashMap userHM   = new HashMap();
    private HashMap userEmailHM   = new HashMap();

	/** the file for persistence */
    private String FileName = null;
    private String SharedFileName = null; 

    private transient SharedCalendarDB sharedDB = null;


    public UserDB(String fn, String sfn)
    {
		FileName = fn;
		SharedFileName = sfn;
    }

    public Object[] getUserArray(){
		return userEmailHM.values().toArray();
    }

    /**
       creates a new user and adds it to the database.
       @return the identifier of the object (a positive integer)
     */
    Integer addUser(User user)
    {
		addUserDB(user);
		return user.getId();
    }

    /**
       creates a new user with a given key and adds it to 
       the database. 
     */
    private void addUser(User user, Integer id)
    {
		user.setId(id);
		addUserDB(user);
    }

    /** inserts the user into the data base.
     */
    private void addUserDB(User user)
    {
		userHM.put(user.getId(),user);
		userEmailHM.put(user.getEmail(),user);
    }


	/** removes an user with the given key. 
	 @returns true if an event with that key existed, otherwise false
	 */
    public boolean deleteUser(User user)
    {
		Object o = userHM.remove(user.getId());

		if (o==null)
		    { // we have no null binding => wrong key
			return false;
		    }
		else
		    {
			o = userEmailHM.remove(user.getEmail());
			return true;
		    }
    }


    /**
       edits user with a given key from the database. 
     */
    void editUser(User user)
    {
		if (deleteUser(user)) {
			User newUser = user.copy();
			addUserDB(newUser);
			}
	}


    /**
     */
    User getUser(String email)
    {
		User user = (User) userEmailHM.get(email);
		return (user==null)?user:(user.copy());
    }

    User getUser(Integer id)
    {
		User user = (User) userHM.get(id);
		return (user==null)?user:(user.copy());
    }


    public void saveModifiedCalendars(){
		Collection col = userHM.values();
		for(Iterator it = col.iterator(); it.hasNext();) {
		    User user=(User)it.next();
		    CalendarReference cref=user.getCalendar();
		    if(cref.isModified())
			cref.getCalendarDB().save();
		}
		sharedDB.save();
    }

    public SharedCalendarDB  getSharedCalendar(){
		if(sharedDB == null){
		    sharedDB = new SharedCalendarDB(SharedFileName,this);
		    sharedDB.restore();
		}
		return sharedDB;
    }

    public void save(){
      //Write to file 
         try { 
          FileOutputStream out = new FileOutputStream(FileName); 
          ObjectOutputStream oout = new ObjectOutputStream(out); 

          
          oout.writeObject(userHM);
          oout.writeObject(userEmailHM);

	  oout.flush();
          out.close(); 
         } catch(java.io.IOException e) { 
           System.out.println(e.toString()); 
         } 

	}

	/** restore the database from a default file (see Definition.java)
	 
	 */
	public void restore(){
      //Read from file 
          try { 
           FileInputStream in = new FileInputStream(FileName); 
           ObjectInputStream oin = new ObjectInputStream(in); 

           
           userHM = (HashMap)oin.readObject(); 
           userEmailHM = (HashMap)oin.readObject(); 

           in.close(); 
          } catch(java.io.IOException e) { 
            System.out.println("Cannot read from "+FileName); 
          }  catch(java.lang.ClassNotFoundException e) { 
            System.out.println(e.toString()); 
          } 
	}

}
