package coolCalendar;

import java.util.*;
import java.text.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;

/**
 * A dialog box to add/edit users properties.
 */
public class UserEditDialog 
    extends JDialog
{
    private JPanel mainPanel = null;

    private JTextField emailField  = null;
    private JTextField firstNameField  = null;
    private JTextField lastNameField  = null;
    private JPasswordField password1Field = null;
    private JPasswordField password2Field = null;

    private JButton okayButton = null;
    private JButton cancelButton = null;

    private boolean result = false;
    private boolean addingUser = false;
    private UserDB myUserDB = null;


    /**
     * Construct a dialog.
     */
    public UserEditDialog(Frame parent, boolean isAddUser, String frameName, UserDB userDatabase)
    {
        super(parent, frameName, true);

        addingUser = isAddUser;
        myUserDB = userDatabase;

		// Dialog is not resizable.
        setResizable(false);
		setSize(300, 250);
		setModal(true);
		mainPanel = new JPanel() {
    		public  void paintComponent(Graphics g) {
				if (g != null) {
					ImageIcon tmpImage = new ImageIcon(Definition.AddNewUserImage);
					g.drawImage(tmpImage.getImage(), 0, 0, mainPanel.getWidth(), mainPanel.getHeight(), this);
					super.paintComponent(g);
				}
			}};
		mainPanel.setOpaque(false);

        // Set close handler for window.
        setDefaultCloseOperation(JDialog.DO_NOTHING_ON_CLOSE);
        addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent event)
            {
                doCancel();
            }
        });


        // Layout components.
        getContentPane().add(mainPanel);
        mainPanel.setLayout(new BorderLayout());
        mainPanel.add(createEditPanel(), BorderLayout.NORTH);
        mainPanel.add(createButtonPanel(), BorderLayout.SOUTH);
    }


    /**
     * Return the result of the dialog.
     */
    public boolean isOkay()
    {
        return result;
    }


    /**
     * Return the user.
     */
    public User getUser()
    {
        return new User(emailField.getText(), new String(password1Field.getPassword()),
			firstNameField.getText(), lastNameField.getText(), null);
    }

    /**
     * Set the user data.
     */
    public void setUser(User u)
    {
        emailField.setText(u.getEmail());
        password1Field.setText(u.getPassword());
        password2Field.setText(u.getPassword());
        firstNameField.setText(u.getFirstName());
        lastNameField.setText(u.getLastName());
    }


    /**
     * Return the email.
     */
    public String getEmail()
    {
        return emailField.getText();
    }

    /**
     * Set the email
     */
    public void setEmail(String email)
    {
        emailField.setText(email);
    }


    /**
     * Return the password
     */
    public String getPassword()
    {
        return new String(password1Field.getPassword());
    }

    /**
     * Set the password
     */
    public void setpassword(String password)
    {
        password1Field.setText(password);
        password2Field.setText(password);
    }


    /**
     * Return the firstName.
     */
    public String getfirstName()
    {
        return firstNameField.getText();
    }

    /**
     * Set the firstName.
     */
    public void setfirstName(String fn)
    {
        firstNameField.setText(fn);
    }


    /**
     * Return the lastName.
     */
    public String getLastName()
    {
        return lastNameField.getText();
    }

    /**
     * Set the lastName.
     */
    public void setLastName(String ln)
    {
        lastNameField.setText(ln);
    }


    /**
     * Closes the dialog with "true" result.
     */
    protected void doOkay()
    {
		result = isEmailOK() && isPasswordOK();

		if (result)
		{
			setVisible(false);
			dispose();
		}
	}

    /**
     * Closes the dialog with "false" result.
     */
    protected void doCancel()
    {
        result = false;
        setVisible(false);
        dispose();
    }

    /**
     * Check if Email (login) identifier exists in the user database.
     @ returns true if Email is accepted by the current context (add/edit).
     */
    protected boolean isEmailOK()
    {
    	if (emailField.getText().length() == 0)
    		return false;

    	if ((myUserDB.getUser(emailField.getText()) != null) && addingUser) {
			JOptionPane.showMessageDialog(this,
				"Your desired login identifier is allready in use!"+
				"\nYou must pickup another name!\nPlease try again!",
				"Login identifier", JOptionPane.ERROR_MESSAGE);
			return false;
			}
		return true;
	}

    /**
     * Check if the two passwords match.
     @ returns true if passwords match.
     */
    protected boolean isPasswordOK()
    {
    	if ((new String(password1Field.getPassword()).length() == 0) ||
	    	(new String(password1Field.getPassword()).length() == 0))
    			return false;

		if (!(new String(password1Field.getPassword()).equals(
			new String(password2Field.getPassword())))) {
			JOptionPane.showMessageDialog(this,
				"Password not mached!\nYou must type same password in"+
				"\nthe two password edits!\nPlease try again!",
				"Password", JOptionPane.ERROR_MESSAGE);
			return false;
			}
		return true;
	}

    /**
     * Creates the edit panel.
     */
    protected JPanel createEditPanel()
    {
		JPanel panel = new JPanel(new GridLayout(5,2));
		panel.setOpaque(false);

		JLabel ulabel, p1label, p2label, fnlabel, lnlabel;

		panel.add(ulabel = new JLabel("User:"));
		ulabel.setForeground(Color.white);
		ulabel.setHorizontalAlignment(JLabel.CENTER);
		panel.add(emailField = new JTextField(20) {
            public void actionPerformed(ActionEvent event)
            {
            	if (!isEmailOK())
            		return;
                password1Field.requestFocus();
            }
        });

		panel.add(p1label = new JLabel("Password:"));
		p1label.setForeground(Color.white);
		p1label.setHorizontalAlignment(JLabel.CENTER);
		panel.add(password1Field = new JPasswordField(20) {
            public void actionPerformed(ActionEvent event)
            {
                password2Field.requestFocus();
            }
		});

		panel.add(p2label = new JLabel("Retype password:"));
		p2label.setForeground(Color.white);
		p2label.setHorizontalAlignment(JLabel.CENTER);
		panel.add(password2Field = new JPasswordField(20) {
            public void actionPerformed(ActionEvent event)
            {
            	if (!isPasswordOK())
            		return;
                firstNameField.requestFocus();
            }
		});

		panel.add(fnlabel = new JLabel("First name:"));
		fnlabel.setForeground(Color.white);
		fnlabel.setHorizontalAlignment(JLabel.CENTER);
		panel.add(firstNameField = new JTextField(20) {
            public void actionPerformed(ActionEvent event)
            {
                lastNameField.requestFocus();
            }
		});

		panel.add(lnlabel = new JLabel("Last name:"));
		lnlabel.setForeground(Color.white);
		lnlabel.setHorizontalAlignment(JLabel.CENTER);
		panel.add(lastNameField = new JTextField(20));

		return panel;
    }


    /**
     * Creates the button panel.
     */
    protected JPanel createButtonPanel()
    {
        JPanel panel = new JPanel();
		panel.setOpaque(false);
		String okText = null;
		if (addingUser)
			okText = new String("Add");
		else
			okText = new String("Okay");

        panel.add(okayButton = new JButton(okText));
        panel.add(cancelButton = new JButton("Cancel"));

        // Add some action listeners.
        okayButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent event)
            {
                doOkay();
            }
        });

        cancelButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent event)
            {
                doCancel();
            }
        });

        return panel;
    }
}
