package coolCalendar;

import java.util.*;
import java.io.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.border.*;

/**
 * Main Panel
 */
public class UserPanel extends JPanel
{
    private final JButton loginButton=new JButton("Login");
    private final JButton logoutButton = new JButton("Logout");
    private final JButton addButton = new JButton("Add");
    private final JButton removeButton = new JButton("Remove");
    private final JButton editButton = new JButton("Edit");
    private final JButton quitButton = new JButton("Quit");

    private final JPanel buttonPanel = new JPanel(new GridLayout(1, 6));

    private UserJList userLoginList=null;
    private JScrollPane userLoginPane;

    private MutableCalendarPanel panel=null;
    private MultiCalendarModel model=null;
    private CalendarDB cdb=null;

    final private JPanel thisPanel = this; 
    final private JPanel northPanel = new JPanel(new BorderLayout()) ; 
    private Main frame = null;

    private Border lineBorder = null;

    private UserDB userDB = null;
    private SharedCalendarDB sharedDB = null;

    private String lastLogin=null;

    /**
     * Constructs a UserPanel 
     */
    public UserPanel(UserDB udb, Main fr)
    {
        super();

	userDB = udb;
        sharedDB = userDB.getSharedCalendar();
	frame = fr;

	// Set layout manager.
        setLayout(new BorderLayout());

	logoutButton.setEnabled(false);
	
        buttonPanel.add(quitButton);
        buttonPanel.add(loginButton);
        buttonPanel.add(logoutButton);
        buttonPanel.add(addButton);
        buttonPanel.add(editButton );
        buttonPanel.add(removeButton);

	lineBorder = BorderFactory.createLineBorder(Color.blue, 2);
	northPanel.add(buttonPanel, BorderLayout.NORTH);
	northPanel.setBorder(lineBorder);
	
	createUserLoginList();
	userLoginPane=new JScrollPane(userLoginList);
	
	add(northPanel, BorderLayout.NORTH);
	add(userLoginPane, BorderLayout.CENTER);
	
        quitButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent event)
            {
            	frame.closeMainWindow();
            }
        });

        loginButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent event)
            {
				User user = checkUserLogin();
				if (user == null)
					return;

				loginButton.setEnabled(false);
				addButton.setEnabled(false);
				editButton.setEnabled(false);
				removeButton.setEnabled(false);
				logoutButton.setEnabled(true);

				// Retrieve database and model.

				//CalendarReference cref=user.getCalendar();
				//cdb = cref.getCalendarDB();

				model = new MultiCalendarModel(userDB, user);

				thisPanel.remove(userLoginPane);

				//create and add panel
				panel = new MutableCalendarPanel(model, userDB);
				thisPanel.add(panel, BorderLayout.CENTER);

				frame.pack();
				repaint();
            }
        });

        logoutButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent event)
            {
		    userDB.saveModifiedCalendars();
		    
		    cdb=null;model=null;
		    thisPanel.remove(panel);
		    panel=null;
		  
		    loginButton.setEnabled(true);
		    addButton.setEnabled(true);
		    editButton.setEnabled(true);
		    removeButton.setEnabled(true);
		    logoutButton.setEnabled(false);

		    //createUserLoginList();
		    userLoginList.refreshList(userDB.getUserArray());
		    //userLoginPane=new JScrollPane(userLoginList);
		    thisPanel.add(userLoginPane, BorderLayout.CENTER);
			frame.pack();
		    repaint();
            }
        });

        addButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent event)
            {
		// Show user edit dialog.

                UserEditDialog dlg = new UserEditDialog(
				   JOptionPane.getFrameForComponent(UserPanel.this), true,
							"Add new user", userDB);
                centerWindow(dlg);
                dlg.show();
		
		if (dlg.isOkay()){
		    
		    String email = dlg.getEmail();
		    String password = dlg.getPassword();
		    
		    try{
			FileOutputStream out = new FileOutputStream(Definition.getUserDir(email));
			out.flush();
			out.close();
		    } catch(java.io.IOException e) { 
			System.out.println(e.toString()); 
		    } 
		    
		    CalendarInFile cal= new CalendarInFile(Definition.getUserDir(email));

		    User user= dlg.getUser();
		    user.setCalendar(cal);
		    userDB.addUser(user);
		    userLoginList.refreshList(userDB.getUserArray());
		}
		
            }
        });

        editButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent event)
            {
				User user = checkUserLogin();
				if (user == null)
					return;

				// Show user edit dialog.
                UserEditDialog dlg = new UserEditDialog(
                	JOptionPane.getFrameForComponent(UserPanel.this), false,
                	"Edit user", userDB);
                centerWindow(dlg);
                dlg.setUser(user);
                dlg.show();

				if (dlg.isOkay()){
				    String email = dlg.getEmail();
				    String password = dlg.getPassword();
				    User foundUser = userDB.getUser(dlg.getEmail());
				    if(foundUser == null){
						JOptionPane.showMessageDialog(UserPanel.this,
							"The login you specified has been allready deleted!",
							"User allready deleted", JOptionPane.ERROR_MESSAGE);
						return; 
				    } else {
					    user = dlg.getUser();
					    user.setId(foundUser.getId());
					    userDB.editUser(user);
					}
				}
            }
        });

        removeButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent event)
            {
				User user = checkUserLogin();
				if (user == null)
					return;

				userDB.deleteUser(user);
				userLoginList.refreshList(userDB.getUserArray());
				JOptionPane.showMessageDialog(UserPanel.this,
					"The user " + user + " has been deleted!",
					"Delete", JOptionPane.ERROR_MESSAGE);
            }
        });
    }


	/* show User Menubar NotActive Error
	 */
	void createUserLoginList()
	{
	    
		userLoginList = new UserJList();
		/*
		Object[] ulist = userDB.getUserArray();
		
		String[] slist= new String[ulist.length];
		
		for (int i=0;i<ulist.length;i++)
			slist[i]=((User)ulist[i]).getEmail();
		*/

		userLoginList.refreshList(userDB.getUserArray());

		MouseListener mouseListener = new MouseAdapter() {
		public void mouseClicked(MouseEvent e) {
		    if (e.getClickCount() >= 1) {
			int index = userLoginList.locationToIndex(e.getPoint());
			if (index != -1) {
			userLoginList.setSelectedIndex(index);
			lastLogin=((User)userLoginList.getSelectedValue()).getEmail();
			}
		    }
		    if (e.getClickCount() == 2)
			loginButton.doClick();
		}
			
		};
		userLoginList.addMouseListener(mouseListener);
	}

     /* Utility login function.
      */
    public User checkUserLogin()
    {
    	User user = null;
		// Show authentification dialog.
		AuthentificationDialog dlg = new AuthentificationDialog(
			JOptionPane.getFrameForComponent(UserPanel.this));
		dlg.setEmail(lastLogin);
		centerWindow(dlg);

		dlg.show();

		// If the user clicked okay, then try to log in
		if (dlg.isOkay())
			{
				String email = dlg.getEmail();
				String password = dlg.getPassword();
				user = userDB.getUser(email);

				if(user == null) {
					JOptionPane.showMessageDialog(UserPanel.this,
						"The login you specified doesn't exist!"+
						"\nThe user has NOT been created!",
						"Invalid User Login", JOptionPane.ERROR_MESSAGE);
					return null; 
				}

				if(!password.equals(user.getPassword())) {
					JOptionPane.showMessageDialog(UserPanel.this,
						"Wrong password! Please try again!",
						"Invalid Password", JOptionPane.ERROR_MESSAGE);
					return null;
				}
			}
		return user;
	}

     /* Utility function to center window.
     */
    public void centerWindow(Component window)
    {
        Toolkit toolkit = Toolkit.getDefaultToolkit();
        Dimension dim = toolkit.getScreenSize();
        int x = (dim.width - window.getSize().width) / 2;
        int y = (dim.height - window.getSize().height) / 2;
        window.setLocation(x, y);
    }

}
